@@ -87,6 +87,10 @@ def wvt_conv_flatten(tensor_shape):
   return weight_to_var, var_to_weight, var_shape
 
 
+def wvt_identity(tensor_shape):
+  return lambda X_tensor: X_tensor, lambda X_var: X_var, tensor_shape
+
+
 
 
 
