@@ -41,7 +41,7 @@ class RNNBase_cdopt(Module):
     def __init__(self, mode: str, input_size: int, hidden_size: int,
                  num_layers: int = 1, bias: bool = True, batch_first: bool = False,
                  dropout: float = 0., bidirectional: bool = False, proj_size: int = 0,
-                 device=None, dtype=None, manifold_class = euclidean_torch, weight_var_transfer = None, **kwargs) -> None:
+                 device=None, dtype=None, manifold_class = euclidean_torch, penalty_param = 0, weight_var_transfer = None, manifold_args = {}) -> None:
         factory_kwargs = {'device': device, 'dtype': dtype}
         super(RNNBase_cdopt, self).__init__()
         self.mode = mode
@@ -55,6 +55,8 @@ class RNNBase_cdopt(Module):
         self.proj_size = proj_size
         num_directions = 2 if bidirectional else 1
 
+        self.manifold_args = manifold_args
+        self.penalty_param = penalty_param
 
         
 
@@ -143,7 +145,7 @@ class RNNBase_cdopt(Module):
 
         self.weight_to_var, self.var_to_weight, self.var_shape = weight_var_transfer( (self.hidden_size, self.hidden_size) )
 
-        self.manifold = manifold_class(self.var_shape,  device= device, dtype= dtype,**kwargs)
+        self.manifold = manifold_class(self.var_shape,  device= device, dtype= dtype,**self.manifold_args)
         
 
 
@@ -192,6 +194,13 @@ class RNNBase_cdopt(Module):
     def C(self, _flat_wights):
         return [ self.manifold.C(self.weight_to_var(_flat_wights[i])) for i in  self.cdopt_weight_hh_index]
 
+
+    def feasibility(self):
+        sum_tol = 0
+        for C_weight in self.C(self._flat_weights):
+            sum_tol = sum_tol + torch.sum(C_weight **2 ) 
+        return torch.sqrt(sum_tol)
+
     def quad_penalty(self):
         sum_tol = 0
         for C_weight in self.C(self._flat_weights):
@@ -1029,7 +1038,7 @@ class RNNCellBase_cdopt(Module):
     # See https://github.com/pytorch/pytorch/issues/39670
 
     def __init__(self, input_size: int, hidden_size: int, bias: bool, num_chunks: int,
-                 device=None, dtype=None, manifold_class = euclidean_torch, weight_var_transfer = None, **kwargs) -> None:
+                 device=None, dtype=None, manifold_class = euclidean_torch, penalty_param = 0, weight_var_transfer = None, manifold_args = {}) -> None:
         factory_kwargs = {'device': device, 'dtype': dtype}
         super(RNNCellBase_cdopt, self).__init__()
         self.input_size = input_size
@@ -1056,10 +1065,15 @@ class RNNCellBase_cdopt(Module):
 
         self.weight_to_var, self.var_to_weight, self.var_shape = weight_var_transfer( ( num_chunks * self.hidden_size, self.hidden_size) )
 
+        self.penalty_param = penalty_param
+        self.manifold_args = manifold_args
 
-        self.manifold = manifold_class(self.var_shape, device= device, dtype= dtype, **kwargs)
+
+        self.manifold = manifold_class(self.var_shape, device= device, dtype= dtype, **self.manifold_args)
         self.A = lambda X_tensor: self.var_to_weight(self.manifold.A( self.weight_to_var(X_tensor) ))
         self.C = lambda X_tensor: self.manifold.C( self.weight_to_var(X_tensor) )
+
+        self.feasibility = lambda : torch.linalg.norm(self.C(self.weight_hh).flatten())
         self.quad_penalty = lambda : torch.sum(self.C(self.weight_hh)**2)
 
 
@@ -1135,9 +1149,9 @@ class RNNCell_cdopt(RNNCellBase_cdopt):
     nonlinearity: str
 
     def __init__(self, input_size: int, hidden_size: int, bias: bool = True, nonlinearity: str = "tanh",
-                 device=None, dtype=None, manifold_class = euclidean_torch, **kwargs) -> None:
+                 device=None, dtype=None, manifold_class = euclidean_torch, penalty_param = 0, manifold_args = {}) -> None:
         factory_kwargs = {'device': device, 'dtype': dtype}
-        super(RNNCell_cdopt, self).__init__(input_size, hidden_size, bias, num_chunks=1, device=device, dtype=dtype, manifold_class= manifold_class, **kwargs )
+        super(RNNCell_cdopt, self).__init__(input_size, hidden_size, bias, num_chunks=1, device=device, dtype=dtype, manifold_class= manifold_class, penalty_param = penalty_param, manifold_args = manifold_args )
         self.nonlinearity = nonlinearity
 
     def forward(self, input: Tensor, hx: Optional[Tensor] = None) -> Tensor:
@@ -1224,9 +1238,10 @@ class LSTMCell_cdopt(RNNCellBase_cdopt):
     """
 
     def __init__(self, input_size: int, hidden_size: int, bias: bool = True,
-                 device=None, dtype=None, manifold_class = euclidean_torch, **kwargs) -> None:
+                 device=None, dtype=None, manifold_class = euclidean_torch, penalty_param = 0, manifold_args = {}) -> None:
         factory_kwargs = {'device': device, 'dtype': dtype}
-        super(LSTMCell_cdopt, self).__init__(input_size, hidden_size, bias, num_chunks=4,  device=device, dtype=dtype, manifold_class= manifold_class, **kwargs)
+        super(LSTMCell_cdopt, self).__init__(input_size, hidden_size, bias, num_chunks=4,  device=device, dtype=dtype, 
+        manifold_class= manifold_class, penalty_param = penalty_param, manifold_args = manifold_args)
 
     def forward(self, input: Tensor, hx: Optional[Tuple[Tensor, Tensor]] = None) -> Tuple[Tensor, Tensor]:
         assert input.dim() in (1, 2), \
@@ -1303,9 +1318,10 @@ class GRUCell_cdopt(RNNCellBase_cdopt):
     """
 
     def __init__(self, input_size: int, hidden_size: int, bias: bool = True,
-                 device=None, dtype=None,  manifold_class = euclidean_torch, **kwargs) -> None:
+                 device=None, dtype=None,  manifold_class = euclidean_torch, penalty_param = 0, manifold_args ={}) -> None:
         factory_kwargs = {'device': device, 'dtype': dtype}
-        super(GRUCell_cdopt, self).__init__(input_size, hidden_size, bias, num_chunks=3, device=device, dtype=dtype, manifold_class= manifold_class, **kwargs)
+        super(GRUCell_cdopt, self).__init__(input_size, hidden_size, bias, num_chunks=3, device=device, dtype=dtype, 
+        manifold_class= manifold_class, penalty_param=penalty_param, manifold_args=manifold_args)
 
     def forward(self, input: Tensor, hx: Optional[Tensor] = None) -> Tensor:
         assert input.dim() in (1, 2), \
